/**
 * -----------------------------------------------------
 * File        stamm_adminflag.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Includes
#include <sourcemod>
#include <autoexecconfig>

#undef REQUIRE_PLUGIN
#include <stamm>

#pragma semicolon 1
#pragma newdecls required


public Plugin myinfo =
{
	name = "Stamm Feature Admin Flags",
	author = "Popoklopsi, RAYs3T",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give VIP's Admin Flags",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};




// Add feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable())
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}

	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP Admin Flags", false);
}




// Feature loaded
public void STAMM_OnFeatureLoaded(const char[] basename)
{
	char theflags[64];

	// Get flags for each level
	for (int i=1; i <= STAMM_GetLevelCount(); i++)
	{
		Format(theflags, sizeof(theflags), "");

		getLevelFlag(theflags, sizeof(theflags), i);
	}
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	STAMM_GetBlockName(block, fmt, sizeof(fmt));
	Format(fmt, sizeof(fmt), "%T", "GetAdminFlags", client, fmt);
	
	PushArrayString(array, fmt);
}




public void STAMM_OnClientReady(int client)
{
	if (STAMM_IsClientValid(client))
	{
		char theflags[64];
		int bytes;


		// Get Flags for client level
		Format(theflags, sizeof(theflags), "");
		getClientFlags(client, theflags, sizeof(theflags));


		if (!StrEqual(theflags, "") && theflags[0] != '\0') 
		{
			// Get bits of the string
			bytes = ReadFlagString(theflags);

			// Set flags
			if (bytes)
			{
				SetUserFlagBits(client, bytes | GetUserFlagBits(client));
			}
		}
	}
}




// Give VIP's flags again on rebuild
public void OnRebuildAdminCache(AdminCachePart part)
{
	if (part == AdminCache_Admins)
	{
		for (int client=0; client <= MaxClients; client++)
		{
			if (STAMM_IsClientValid(client))
			{
				char theflags[64];
				int bytes;


				// Get Flags for client level
				Format(theflags, sizeof(theflags), "");
				getClientFlags(client, theflags, sizeof(theflags));


				if (!StrEqual(theflags, "") && theflags[0] != '\0')
				{
					// Get bits of the string
					bytes = ReadFlagString(theflags);

					// Set flags
					if (bytes)
					{
						SetUserFlagBits(client, bytes | GetUserFlagBits(client));
					}
				}
			}
		}
	}
}




public void STAMM_OnClientBecomeVip(int client, int oldlevel, int newlevel)
{
	STAMM_OnClientReady(client);
}




// Read out the flags
public void getLevelFlag(char[] theflags, int size, int level)
{
	int blocks = STAMM_GetBlockCount();


	// Do we have a file?
	if (!FileExists("cfg/stamm/features/adminflags.txt") && blocks <= 0)
	{
		SetFailState("Couldn't find any block and also 'cfg/stamm/features/adminflags.txt' doesn't exist!");

		return;
	}


	if (blocks > 0)
	{
		for (int i=1; i <= blocks; i++)
		{
			if (STAMM_GetBlockLevel(i) == level)
			{
				STAMM_GetBlockName(i, theflags, size);

				break;
			}
		}
	}
	else
	{
		Handle flagvalue = CreateKeyValues("AdminFlags");
		
		FileToKeyValues(flagvalue, "cfg/stamm/features/adminflags.txt");
		

		STAMM_WriteToLog(false, "Attention: Found only old AdminFlags level config. int one is in: cfg/stamm/levels/adminflags.txt!");


		// Key Value loop
		if (KvGotoFirstSubKey(flagvalue, false))
		{
			do
			{
				char section[64];
				
				KvGetSectionName(flagvalue, section, sizeof(section));


				// Only Flags for specific level
				if (STAMM_GetLevelNumber(section) == level)
				{
					KvGoBack(flagvalue);
					KvGetString(flagvalue, section, theflags, size);
				}
			}
			while (KvGotoNextKey(flagvalue, false));
		}
		
		CloseHandle(flagvalue);
	}
}



// Read out the flags
public void getClientFlags(int client, char[] theflags, int size)
{
	if (STAMM_GetBlockCount() > 0)
	{
		int block = STAMM_GetClientBlock(client);

		if (block > 0)
		{
			STAMM_GetBlockName(block, theflags, size);
		}
	}
	else
	{
		Handle flagvalue = CreateKeyValues("AdminFlags");
		
		FileToKeyValues(flagvalue, "cfg/stamm/features/adminflags.txt");
		

		// Key Value loop
		if (KvGotoFirstSubKey(flagvalue, false))
		{
			do
			{
				char section[64];
				
				KvGetSectionName(flagvalue, section, sizeof(section));


				// Only Flags for specific level
				if (STAMM_GetLevelNumber(section) == STAMM_GetClientLevel(client))
				{
					KvGoBack(flagvalue);
					KvGetString(flagvalue, section, theflags, size);
				}
			}
			while (KvGotoNextKey(flagvalue, false));
		}
		
		CloseHandle(flagvalue);
	}
}