/**
 * -----------------------------------------------------
 * File        stamm_end_immunity.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */



// Includes
#include <sourcemod>

#undef REQUIRE_EXTENSIONS
#include <tf2_stocks>
#define REQUIRE_EXTENSIONS

#include <sdktools>

#undef REQUIRE_PLUGIN
#include <stamm>


#pragma semicolon 1
#pragma newdecls required



bool g_bRoundEnd;
int g_iParticels[MAXPLAYERS + 1][2];



public Plugin myinfo =
{
	name = "Stamm Feature End of Round Immunity",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give VIP's immunity at the end of the round",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};



public void STAMM_OnFeatureLoaded(const char[] basename)
{
}



// Hooke needed events
public void OnPluginStart()
{
	HookEvent("teamplay_round_win", RoundWin);
	
	HookEvent("teamplay_round_start", RoundStart);
	HookEvent("teamplay_round_stalemate", RoundStart);
	
	HookEvent("player_death", PlayerDeath);
}



// Add feature for TF2
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}

	if (STAMM_GetGame() != GameTF2) 
	{
		SetFailState("Can't Load Feature, not Supported for your game!");
	}
	
	
	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP End of Round Immunity");
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	Format(fmt, sizeof(fmt), "%T", "GetImmunity", client);
	
	PushArrayString(array, fmt);
}



// A round is finish
public void RoundWin(Handle event, const char[] name, bool dontBroadcast)
{	
	g_bRoundEnd = true;

	// Set god mod for each VIP
	for (int i=1; i <= MaxClients; i++)
	{
		if (STAMM_IsClientValid(i) && STAMM_HaveClientFeature(i))
		{
			SetEntProp(i, Prop_Data, "m_takedamage", 1, 1);
			
			// Show effects
			ImmuneEffects(i);
		}
	}
}



// A round started
public void RoundStart(Handle event, const char[] name, bool dontBroadcast)
{	
	g_bRoundEnd = false;
	
	// Delete Effects
	for (int i=0; i < MAXPLAYERS+1; i++)
	{
		ClearParticles(i);
	}
}



// A player died
public Action PlayerDeath(Handle event, const char[] name, bool dontBroadcast)
{
	if (!g_bRoundEnd) 
	{
		return Plugin_Continue;
	}

	int client = GetClientOfUserId(GetEventInt(event, "userid"));
	

	// Delete effects
	if (STAMM_IsClientValid(client))
	{
		ClearParticles(client);
	}

	return Plugin_Continue;
}




// Create the effects
public void ImmuneEffects(int client)
{
	g_iParticels[client][0] = EntIndexToEntRef(AttachParticle(client, "player_recent_teleport_red", 2.0));
	g_iParticels[client][1] = EntIndexToEntRef(AttachParticle(client, "player_recent_teleport_blue", 2.0));
}



// Attach the effects
public int AttachParticle(int entity, char[] particleType, float offsetZ)
{
	int particle = CreateEntityByName("info_particle_system");
	

	// Is a valid ent?
	if (IsValidEntity(particle))
	{
		float pos[3];

		// Set Origin to client origin
		GetEntPropVector(entity, Prop_Send, "m_vecOrigin", pos);

		pos[2] += offsetZ;
		

		// Teleport it
		TeleportEntity(particle, pos, NULL_VECTOR, NULL_VECTOR);
		

		// Spawn it
		DispatchKeyValue(particle, "effect_name", particleType);
		
		DispatchSpawn(particle);
		

		// Set parent to client
		SetVariantString("!activator");
		AcceptEntityInput(particle, "SetParent", entity);
		
		ActivateEntity(particle);
		
		AcceptEntityInput(particle, "start");
		
		return particle;
	}
	
	return -1;
}



// Clear effects
public void ClearParticles(int client)
{
	// Client have effect?
	if (g_iParticels[client][0] > 0)
	{
		int particle = EntRefToEntIndex(g_iParticels[client][0]);
		
		// Kill
		if (particle > MaxClients && IsValidEntity(particle))
		{
			AcceptEntityInput(particle, "Kill");
		}

		g_iParticels[client][0] = 0;
	}

	if (g_iParticels[client][1] > 0)
	{
		int particle = EntRefToEntIndex(g_iParticels[client][1]);
		
		// Kill
		if (particle > MaxClients && IsValidEntity(particle))
		{
			AcceptEntityInput(particle, "Kill");
		}
			
		g_iParticels[client][1] = 0;
	}
}



// Set speed to highest on Round end
public void OnGameFrame()
{
	if (g_bRoundEnd)
	{
		// For each VIP
		for (int i=1; i <= MaxClients; i++)
		{
			if (STAMM_IsClientValid(i) && IsPlayerAlive(i))
			{
				TFClassType class = TF2_GetPlayerClass(i);
				
				if (class != TFClass_Scout && !TF2_IsPlayerInCondition(i, TFCond_Charging) && class != TFClass_Unknown)
				{
					// set Speed
					if (STAMM_HaveClientFeature(i))
					{
						SetEntPropFloat(i, Prop_Send, "m_flMaxspeed", 400.0);
					}
				}
			}
		}
	}
}