/**
 * -----------------------------------------------------
 * File        stamm_froggyjump.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Include
#include <sourcemod>
#include <autoexecconfig>

#undef REQUIRE_PLUGIN
#include <stamm>

// For TF2
#undef REQUIRE_EXTENSIONS
#include <tf2_stocks>



#pragma semicolon 1
#pragma newdecls required


int g_iFroggyJumped[MAXPLAYERS + 1];

Handle  g_hStrong;




public Plugin myinfo =
{
	name = "Stamm Feature FroggyJump",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "VIP's have Froggy Jump",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};



public void STAMM_OnFeatureLoaded(const char[] basename)
{
}




// Create config
public void OnPluginStart()
{
	AutoExecConfig_SetFile("froggyjump", "stamm/features");
	AutoExecConfig_SetCreateFile(true);

	g_hStrong = AutoExecConfig_CreateConVar("froggyjump_strong", "200", "The push up strong");

	AutoExecConfig_CleanFile();
	AutoExecConfig_ExecuteFile();
}





// Add the feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}


	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP FroggyJump");
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	Format(fmt, sizeof(fmt), "%T", "GetFroggyJump", client);
	
	PushArrayString(array, fmt);
}



// Allow double jump
public Action OnPlayerRunCmd(int client, int &buttons, int &impulse, float vel[3], float angles[3], int &weapon)
{
	if (!STAMM_IsClientValid(client))
	{
		return Plugin_Continue;
	}


	// VIP?
	if (!STAMM_HaveClientFeature(client) || !IsPlayerAlive(client))
	{
		return Plugin_Continue;
	}


	// In TF2, not for Scout
	if (STAMM_GetGame() == GameTF2)
	{
		if (TF2_GetPlayerClass(client) == TFClass_Scout)
		{
			return Plugin_Continue;
		}
	}


	// Last button
	static bool bPressed[MAXPLAYERS+1] = false;
	

	// Reset when on Ground
	if (GetEntityFlags(client) & FL_ONGROUND)
	{
		g_iFroggyJumped[client] = 0;
		bPressed[client] = false;
	}
	else
	{
		// Player pressed jump button?
		if (buttons & IN_JUMP)
		{

			// For second time?
			if (!bPressed[client] && g_iFroggyJumped[client]++ == 1)
			{
				float velocity[3];
				float velocity0;
				float velocity1;
				float velocity2;
				float velocity2_new;

				// Get player velocity
				velocity0 = GetEntPropFloat(client, Prop_Send, "m_vecVelocity[0]");
				velocity1 = GetEntPropFloat(client, Prop_Send, "m_vecVelocity[1]");
				velocity2 = GetEntPropFloat(client, Prop_Send, "m_vecVelocity[2]");

				float velocity2_int = float(GetConVarInt(g_hStrong));



				// calculate int velocity^^
				if (velocity2 < 150.0)
				{
					velocity2_int = velocity2_int + 20.0;
				}
				if (velocity2 < 100.0) 
				{
					velocity2_int = velocity2_int + 30.0;
				}
				if (velocity2 < 50.0) 
				{
					velocity2_int = velocity2_int + 40.0;
				}
				if (velocity2 < 0.0) 
				{
					velocity2_int = velocity2_int + 50.0;
				}
				if (velocity2 < -50.0) 
				{
					velocity2_int = velocity2_int + 60.0;
				}
				if (velocity2 < -100.0) 
				{
					velocity2_int = velocity2_int + 70.0;
				}
				if (velocity2 < -150.0) 
				{
					velocity2_int = velocity2_int + 80.0;
				}
				if (velocity2 < -200.0) 
				{
					velocity2_int = velocity2_int + 90.0;
				}



				// Set int velocity
				velocity[0] = velocity0 * 0.1;
				velocity[1] = velocity1 * 0.1;
				velocity[2] = velocity2_new;
				
				// Double Jump
				SetEntPropVector(client, Prop_Send, "m_vecBaseVelocity", velocity);
			}

			bPressed[client] = true;
		}
		else
		{
			bPressed[client] = false;
		}
	}

	return Plugin_Continue;
}