/**
 * -----------------------------------------------------
 * File        stamm_high_firingrate.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Includes
#include <sourcemod>
#include <autoexecconfig>

#undef REQUIRE_EXTENSIONS
#include <tf2items>
#define REQUIRE_EXTENSIONS

#undef REQUIRE_PLUGIN
#include <stamm>

#pragma semicolon 1
#pragma newdecls required


Handle  g_hFireRate;




public Plugin myinfo =
{
	name = "Stamm Feature Higher Firing Rate",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give VIP's higher firing Rate",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};



// Add feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}

	if (STAMM_GetGame() != GameTF2) 
	{
		SetFailState("Can't Load Feature, not Supported for your game!");
	}


	// We need tf2items
	if (GetExtensionFileStatus("tf2items.ext") != 1)
	{
		SetFailState("Can't Load Feature, you need to install tf2items!");
	}



	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP Higher Firing Rate");
}




// Create config
public void OnPluginStart()
{
	AutoExecConfig_SetFile("higher_firingrate", "stamm/features");
	AutoExecConfig_SetCreateFile(true);

	g_hFireRate = AutoExecConfig_CreateConVar("firing_rate", "10", "Firing rate increase in percent each block!");
	
	AutoExecConfig_CleanFile();
	AutoExecConfig_ExecuteFile();
}



public void STAMM_OnFeatureLoaded(const char[] basename)
{
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	Format(fmt, sizeof(fmt), "%T", "GetHigherFiringRate", client, GetConVarInt(g_hFireRate) * block);
	
	PushArrayString(array, fmt);
}




// A Item gived to player
public Action TF2Items_OnGiveNamedItem(int client, char[] classname, int iItemDefinitionIndex, TF2Item &hItem)
{
	bool change = false;
	
	
	if (STAMM_IsClientValid(client) && IsPlayerAlive(client))
	{
		// Get highest client block
		int clientBlock = STAMM_GetClientBlock(client);


		// Have client block
		if (clientBlock > 0)
		{
			// Create int item
			hItem = TF2Items_CreateItem(OVERRIDE_ALL);
			
			TF2Items_SetItemIndex(hItem, -1);

			float newFire = 1.0 - float(GetConVarInt(g_hFireRate))/100.0 * clientBlock;


			if (newFire < 0.1)
			{
				newFire = 0.1;
			}


			// Set int firing rate of item
			TF2Items_SetAttribute(hItem, 0, 6, newFire);
				
			// Override old
			TF2Items_SetNumAttributes(hItem, 1);
			TF2Items_SetFlags(hItem, OVERRIDE_ATTRIBUTES|PRESERVE_ATTRIBUTES);
			
			change = true;
		}
	}
	
	if (change)
	{
		return Plugin_Changed;
	}
		
	return Plugin_Continue;
}