/**
 * -----------------------------------------------------
 * File        stamm_moreammo.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Includes
#include <sourcemod>
#include <sdktools>
#include <autoexecconfig>

#undef REQUIRE_PLUGIN
#include <stamm>

#pragma semicolon 1
#pragma newdecls required





Handle  g_hAmmo;
Handle  g_hTheTimer;

bool g_bWeaponEdit[MAXPLAYERS + 1][2024];




public Plugin myinfo =
{
	name = "Stamm Feature MoreAmmo",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give VIP's more ammo",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};




// Add feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}

	if (STAMM_GetGame() == GameTF2)
	{
		HookEvent("teamplay_round_start", RoundStart);
		HookEvent("arena_round_start", RoundStart);
	}
	

	if (STAMM_GetGame() == GameDOD)
	{
		HookEvent("dod_round_start", RoundStart);
	}

	else
	{
		HookEvent("round_start", RoundStart);
	}

	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP MoreAmmo");
}





// Create config and hook round start
public void OnPluginStart()
{
	HookEvent("player_death", PlayerDeath);


	// Config
	AutoExecConfig_SetFile("moreammo", "stamm/features");
	AutoExecConfig_SetCreateFile(true);

	g_hAmmo = AutoExecConfig_CreateConVar("ammo_amount", "20", "Ammo increase in percent each block!");
	
	AutoExecConfig_CleanFile();
	AutoExecConfig_ExecuteFile();
}


public void STAMM_OnFeatureLoaded(const char[] basename)
{
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	Format(fmt, sizeof(fmt), "%T", "GetMoreAmmo", client, GetConVarInt(g_hAmmo) * block);
	
	PushArrayString(array, fmt);
}




// Reset on mapstart
public void OnMapStart()
{
	if (g_hTheTimer != INVALID_HANDLE) 
	{
		KillTimer(g_hTheTimer);
	}

	// Create check timer
	g_hTheTimer = CreateTimer(1.0, CheckWeapons, _, TIMER_REPEAT);
}




// Reset on death
public void PlayerDeath(Handle event, char[] name, bool dontBroadcast)
{
	int client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	for (int x=0; x < 2024; x++) 
	{
		g_bWeaponEdit[client][x] = false;
	}
}




public void RoundStart(Handle event, char[] name, bool dontBroadcast)
{
	// Reset on round start
	for (int x=0; x < 2024; x++)
	{
		for (int i=0; i <= MaxClients; i++) 
		{
			g_bWeaponEdit[i][x] = false;
		}
	}
}




// Check weapons
public Action CheckWeapons(Handle timer, any data)
{
	int ammo = GetConVarInt(g_hAmmo);

	// Client loop
	for (int i = 1; i <= MaxClients; i++)
	{
		int client = i;
		
		// Client valid?
		if (STAMM_IsClientValid(client) && IsPlayerAlive(client) && (GetClientTeam(client) == 2 || GetClientTeam(client) == 3))
		{
			// Get highest client block
			int clientBlock = STAMM_GetClientBlock(client);


			// Client have block?
			if (clientBlock > 0)
			{
				// Weapon loop
				for (int x=0; x < 2; x++)
				{
					// Player carry weapon?
					int weapon = GetPlayerWeaponSlot(client, x);

					if (weapon != -1 && !g_bWeaponEdit[client][weapon])
					{
						// Get ammo index
						int ammotype = GetEntProp(weapon, Prop_Send, "m_iPrimaryAmmoType");

						// Found ammo?
						if (ammotype != -1)
						{
							// Get ammo count
							int cAmmo = GetEntProp(client, Prop_Send, "m_iAmmo", _, ammotype);
							
							// Found ammo count
							if (cAmmo > 0)
							{
								// Calculate int Ammo
								int newAmmo;
								
								newAmmo = RoundToZero(cAmmo + ((float(cAmmo)/100.0) * (clientBlock * ammo)));
								
								// Set ammo
								SetEntProp(client, Prop_Send, "m_iAmmo", newAmmo, _, ammotype);
								
								g_bWeaponEdit[client][weapon] = true;
							}
						}
					}
				}
			}
		}
	}
	
	return Plugin_Continue;
}